#include "dialog.h"

#include <QLabel>
#include <QLineEdit>
#include <QComboBox>
#include <QSpinBox>
#include <QPushButton>
#include <QGridLayout>

#include <QtSerialPort/QSerialPortInfo>

QT_USE_NAMESPACE

// definitions of pixel array
//MCS-12085, ADNS-2610 = 18*18, ADNS5020 15*15
#define	ConstBaudRate	230400	//230400 для arduino nano, 115200 для Digispark
#define ARRAY_WIDTH		18
#define ARRAY_HEIGHT	18

#define MaskBitData		64	//колич. бит в потоке данных (нужен для создания корректной палитры)
							//MCS-12085, ADNS-2610 = 6bit = 64, ADNS5020 = 7bit = 128


unsigned char rawData[ARRAY_WIDTH*ARRAY_HEIGHT];
QImage image(ARRAY_WIDTH, ARRAY_HEIGHT, QImage::Format_Indexed8);

Dialog::Dialog(QWidget *parent)
	: QDialog(parent)
	, transactionCount(0)
	, serialPortLabel(new QLabel(tr("Serial port:")))
	, serialPortComboBox(new QComboBox())
	, waitResponseLabel(new QLabel(tr("Wait response, msec:")))
	, waitResponseSpinBox(new QSpinBox())
	, requestLabel(new QLabel(tr("Request:")))
	, requestLineEdit(new QLineEdit(tr("S")))
	, trafficLabel(new QLabel(tr("No traffic.")))
	, statusLabel(new QLabel(tr("Status: Not running.")))
    , servicePixLabel(new QLabel(tr("Service info")))
	, runButton(new QPushButton(tr("Start")))
{
	foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts())
		serialPortComboBox->addItem(info.portName());

	waitResponseSpinBox->setRange(0, 10000);
	waitResponseSpinBox->setValue(20);

	QGridLayout *mainLayout = new QGridLayout;
	mainLayout->addWidget(serialPortLabel, 0, 0);
	mainLayout->addWidget(serialPortComboBox, 0, 1);
	mainLayout->addWidget(waitResponseLabel, 1, 0);
	mainLayout->addWidget(waitResponseSpinBox, 1, 1);
	mainLayout->addWidget(runButton, 0, 2, 2, 1);
	mainLayout->addWidget(requestLabel, 2, 0);
	mainLayout->addWidget(requestLineEdit, 2, 1, 1, 3);
	mainLayout->addWidget(trafficLabel, 3, 0, 1, 4);
	mainLayout->addWidget(statusLabel, 4, 0, 1, 5);
    mainLayout->addWidget(servicePixLabel, 5, 0, 1, 5);
	setLayout(mainLayout);

	setWindowTitle(tr("Master"));

	//создать палитру для image
	//макс. значение яркости = 63
	QVector<QRgb> palette;
//	for(int i = 0; i < 64; ++i)		palette.append(qRgb(i*(256/64),i*(256/64),i*(256/64)));
	for(int i = 0; i < MaskBitData; ++i)	palette.append(qRgb(i*(256/MaskBitData),i*(256/MaskBitData),i*(256/MaskBitData)));

	image.setColorTable(palette);
/*
	unsigned char tmpByte = 0;
	for(int i = 0; i < ARRAY_WIDTH*ARRAY_HEIGHT; i++)
	{
		rawData[i] = tmpByte;
		tmpByte +=1;
	}

	for (int y = 0; y < image.height(); y++)
	{
		memcpy(image.scanLine(y), rawData + y*ARRAY_WIDTH, image.bytesPerLine());
	}

	QImage scaled = image.scaled(180*2, 180*2); // Scale image to show results better
	statusLabel->setPixmap(QPixmap::fromImage(scaled)); // Show result on a form
*/

	serialPortComboBox->setFocus();

	timer.setSingleShot(true);

	connect(runButton, SIGNAL(clicked()),
			this, SLOT(sendRequest()));
	connect(&serial, SIGNAL(readyRead()),
			this, SLOT(readResponse()));
	connect(&timer, SIGNAL(timeout()),
			this, SLOT(processTimeout()));
}

void Dialog::sendRequest()
{
	if (serial.portName() != serialPortComboBox->currentText())
	{
		serial.close();
		serial.setPortName(serialPortComboBox->currentText());

		serial.setBaudRate(ConstBaudRate);
		serial.setDataBits( QSerialPort::Data8 );
		serial.setParity( QSerialPort::NoParity );
		serial.setStopBits( QSerialPort::OneStop );
		serial.setFlowControl( QSerialPort::NoFlowControl );

		if (!serial.open(QIODevice::ReadWrite))
		{
			processError(tr("Can't open %1, error code %2")
						 .arg(serial.portName()).arg(serial.error()));
			return;
		}
	}

	setControlsEnabled(false);
	statusLabel->setText(tr("Status: Running, connected to port %1.")
						 .arg(serialPortComboBox->currentText()));

	serial.write(requestLineEdit->text().toLocal8Bit());
	timer.start(waitResponseSpinBox->value());
}

void Dialog::readResponse()
{

//	response.append(serial.readBufferSize());

	if (!timer.isActive())
		timer.start(waitResponseSpinBox->value());
	response.append(serial.readAll());

}

void Dialog::processTimeout()
{
	setControlsEnabled(true);

    if(response.size() !=(ARRAY_WIDTH*ARRAY_HEIGHT +7))
	{
		trafficLabel->setText(tr("Traffic, transaction #%1:"
							"\n\r-request: %2"
							"\n\r-response size:%3")
							.arg(++transactionCount).arg(requestLineEdit->text()).arg(response.size()));
	}
	else
	{
		for (int y = 0; y < image.height(); y++)
		{
			memcpy(rawData + y*ARRAY_WIDTH, response.data() + y*ARRAY_WIDTH, image.bytesPerLine());
			memcpy(image.scanLine(y), response.data() + y*ARRAY_WIDTH, image.bytesPerLine());
		}
		statusLabel->setPixmap(QPixmap::fromImage(image.scaled(180*2, 180*2))); // Show result on a form

        servicePixLabel->setText(tr("Max: %1, Min: %2, Ave: %3, Shut: %4, Squal: %5")
                                 .arg((int)*(response.data()+325)) //MAX_PIXEL
                                 .arg((unsigned char)*(response.data()+326)) //MIN_PIXEL
                                 .arg((unsigned char)*(response.data()+327)) //AVE
                                 .arg((unsigned int)(((unsigned char)*(response.data()+328)*256)+((unsigned char)*(response.data()+329))))
                                 .arg((int)*(response.data()+324))); //SQUAL
/*
		trafficLabel->setText("\n\r");
		int nIndex = 0;
		for (int y = 0; y < ARRAY_HEIGHT; ++y)
		{
			// dump one row
			for (int x = 0; x < ARRAY_WIDTH; ++x)
			{
				// format nicely
				if (rawData[nIndex] < 10)
				{
					trafficLabel->setText(trafficLabel->text() + tr("0%1 ").arg(rawData[nIndex++]));
				}
				else
				{
					trafficLabel->setText(trafficLabel->text() + tr("%1 ").arg(rawData[nIndex++]));
				}
			}
			// next row
			trafficLabel->setText(trafficLabel->text() + "\n\r");
		}
*/
	}
	response.clear();
}

void Dialog::processError(const QString &error)
{
	setControlsEnabled(true);
	statusLabel->setText(tr("Status: Not running, %1.").arg(error));
	trafficLabel->setText(tr("No traffic."));
}

void Dialog::setControlsEnabled(bool enable)
{
	runButton->setEnabled(enable);
	serialPortComboBox->setEnabled(enable);
	waitResponseSpinBox->setEnabled(enable);
	requestLineEdit->setEnabled(enable);
}
